using System;
using System.Linq;
using FluentAssertions;
using Newtonsoft.Json;
using NUnit.Framework;

namespace E2ETests.Subscriber;

[Order(200)]
[TestFixture]
public class SubscriberTestHappyPath : SubscriberTestBase {
    [Test]
    public void GetAvailableSubmissions_WithDestinationId_ShouldReturnSubmissionsForPickupDto() {
        // Act
        var submissions = base.subscriber.GetAvailableSubmissions(desitnationId, 0, 100).ToList();

        // Assert
        submissions.Count().Should().BeGreaterThan(0);
        submissions.All(s => s.DestinationId == desitnationId).Should().BeTrue();
    }

    [Test]
    public void GetAvailableSubmissions_WithOutDestinationId_ShouldReturnSubmissionsForPickupDto() {
        // Act
        var submissions = base.subscriber.GetAvailableSubmissions(null, 0, 100).ToList();

        // Assert
        submissions.Count().Should().BeGreaterThan(0);
    }


    [Test]
    public void GetAllSubmission_WithSubmissionId_ShouldReturnSubmissionsForPickupDto() {
        // Arrange
        int errorCounter = 0;
        var submissions = base.subscriber.GetAvailableSubmissions(null, 0, 100).ToList();
        submissions.Count().Should().BeGreaterThan(0);

        foreach (var submissionId in submissions.Select(s => s.SubmissionId)) {
            // var submissionId = "7c7bec6f-84c4-4ab5-9d3e-246b7881c8c3"; // submissions.Last().SubmissionId;

            // Act
            try {
                Console.WriteLine($"Getting submission {submissionId}");
                var dto = subscriber.RequestSubmission(submissionId);

                // Assert
                dto.Submission.Id.Should().Be(submissionId);
            }
            catch (Exception ex) {
                Console.WriteLine($"Exception getting submission {submissionId}");
                errorCounter++;
            }

            errorCounter.Should().BeLessThan(submissions.Count());
            Console.WriteLine($"Error counter: {errorCounter}/{submissions.Count()}");
            /*
             Console.WriteLine(dto.Submission.Data);
             Console.WriteLine(JsonConvert.SerializeObject(dto.Submission.Metadata));
             */
        }
    }

    [TestCase("b0bd846f-f56a-4d06-a586-14fb0ea94fd0")]
    public void GetSingleSubmission_WithSubmissionId_ShouldReturnSubmissionsForPickupDto(
        string submissionId) {
        // Arrange
        var submissions = base.subscriber.GetAvailableSubmissions(null, 0, 100).ToList();
        submissions.Count().Should().BeGreaterThan(0);

        Console.WriteLine($"Getting submission {submissionId}");
        var dto = subscriber.RequestSubmission(submissionId);

        // Assert
        dto.Submission.Id.Should().Be(submissionId);


        Console.WriteLine(dto.Submission.Data);
        Console.WriteLine(JsonConvert.SerializeObject(dto.Submission.Metadata));
    }
}


